@@ -1,23 +1,27 @@
 import inspect
 import warnings
+from collections.abc import Sequence
 from functools import partial, wraps
 from types import FunctionType, GeneratorType
 from typing import (
     Callable,
-    Dict,
-    List,
     Optional,
-    Sequence,
-    Type,
     TypeVar,
     Union,
 )
 
 from superqt.utils import _qthreading
-from typing_extensions import ParamSpec
 
-from ..utils.progress import progress
-from ..utils.translations import trans
+from napari.utils.progress import progress
+from napari.utils.translations import trans
+
+__all__ = [
+    'FunctionWorker',
+    'GeneratorWorker',
+    'create_worker',
+    'thread_worker',
+    'register_threadworker_processors',
+]
 
 wait_for_workers_to_quit = _qthreading.WorkerBase.await_workers
 
@@ -29,30 +33,27 @@ class _NotifyingMixin:
         self.warned.connect(self._relay_warning)
 
     def _relay_error(self, exc: Exception):
-        from ..utils.notifications import notification_manager
+        from napari.utils.notifications import notification_manager
 
         notification_manager.receive_error(type(exc), exc, exc.__traceback__)
 
     def _relay_warning(self, show_warn_args: tuple):
-        from ..utils.notifications import notification_manager
+        from napari.utils.notifications import notification_manager
 
         notification_manager.receive_warning(*show_warn_args)
 
 
-_Y = TypeVar("_Y")
-_S = TypeVar("_S")
-_R = TypeVar("_R")
-_P = ParamSpec("_P")
+_Y = TypeVar('_Y')
+_S = TypeVar('_S')
+_R = TypeVar('_R')
 
 
-class FunctionWorker(_qthreading.FunctionWorker[_R], _NotifyingMixin):
-    ...
+class FunctionWorker(_qthreading.FunctionWorker[_R], _NotifyingMixin): ...
 
 
 class GeneratorWorker(
     _qthreading.GeneratorWorker[_Y, _S, _R], _NotifyingMixin
-):
-    ...
+): ...
 
 
 # these are re-implemented from superqt just to provide progress
@@ -62,10 +63,10 @@ def create_worker(
     func: Union[FunctionType, GeneratorType],
     *args,
     _start_thread: Optional[bool] = None,
-    _connect: Optional[Dict[str, Union[Callable, Sequence[Callable]]]] = None,
-    _progress: Optional[Union[bool, Dict[str, Union[int, bool, str]]]] = None,
+    _connect: Optional[dict[str, Union[Callable, Sequence[Callable]]]] = None,
+    _progress: Optional[Union[bool, dict[str, Union[int, bool, str]]]] = None,
     _worker_class: Union[
-        Type[GeneratorWorker], Type[FunctionWorker], None
+        type[GeneratorWorker], type[FunctionWorker], None
     ] = None,
     _ignore_errors: bool = False,
     **kwargs,
@@ -161,7 +162,7 @@ def create_worker(
         if isinstance(worker, FunctionWorker) and total != 0:
             warnings.warn(
                 trans._(
-                    "_progress total != 0 but worker is FunctionWorker and will not yield. Returning indeterminate progress bar...",
+                    '_progress total != 0 but worker is FunctionWorker and will not yield. Returning indeterminate progress bar...',
                     deferred=True,
                 ),
                 RuntimeWarning,
@@ -196,14 +197,13 @@ def create_worker(
 def thread_worker(
     function: Optional[Callable] = None,
     start_thread: Optional[bool] = None,
-    connect: Optional[Dict[str, Union[Callable, Sequence[Callable]]]] = None,
-    progress: Optional[Union[bool, Dict[str, Union[int, bool, str]]]] = None,
+    connect: Optional[dict[str, Union[Callable, Sequence[Callable]]]] = None,
+    progress: Optional[Union[bool, dict[str, Union[int, bool, str]]]] = None,
     worker_class: Union[
-        Type[FunctionWorker], Type[GeneratorWorker], None
+        type[FunctionWorker], type[GeneratorWorker], None
     ] = None,
     ignore_errors: bool = False,
 ):
-
     """Decorator that runs a function in a separate thread when called.
 
     When called, the decorated function returns a :class:`WorkerBase`.  See
@@ -212,10 +212,10 @@ def thread_worker(
 
     The returned worker will have these signals:
 
-        - *started*: emitted when the work is started
-        - *finished*: emitted when the work is finished
-        - *returned*: emitted with return value
-        - *errored*: emitted with error object on Exception
+    - *started*: emitted when the work is started
+    - *finished*: emitted when the work is finished
+    - *returned*: emitted with return value
+    - *errored*: emitted with error object on Exception
 
     It will also have a ``worker.start()`` method that can be used to start
     execution of the function in another thread. (useful if you need to connect
@@ -224,17 +224,17 @@ def thread_worker(
     If the decorated function is a generator, the returned worker will also
     provide these signals:
 
-        - *yielded*: emitted with yielded values
-        - *paused*: emitted when a running job has successfully paused
-        - *resumed*: emitted when a paused job has successfully resumed
-        - *aborted*: emitted when a running job is successfully aborted
+    - *yielded*: emitted with yielded values
+    - *paused*: emitted when a running job has successfully paused
+    - *resumed*: emitted when a paused job has successfully resumed
+    - *aborted*: emitted when a running job is successfully aborted
 
     And these methods:
 
-        - *quit*: ask the thread to quit
-        - *toggle_paused*: toggle the running state of the thread.
-        - *send*: send a value into the generator.  (This requires that your
-          decorator function uses the ``value = yield`` syntax)
+    - *quit*: ask the thread to quit
+    - *toggle_paused*: toggle the running state of the thread.
+    - *send*: send a value into the generator.  (This requires that your
+        decorator function uses the ``value = yield`` syntax)
 
     Parameters
     ----------
@@ -320,25 +320,53 @@ def thread_worker(
 _new_worker_qthread = _qthreading.new_worker_qthread
 
 
-def _register():
+def _add_worker_data(worker: FunctionWorker, return_type, source=None):
+    from napari._qt._qapp_model.injection._qprocessors import (
+        _add_layer_data_to_viewer,
+    )
+
+    cb = _add_layer_data_to_viewer
+    worker.signals.returned.connect(
+        partial(cb, return_type=return_type, source=source)
+    )
+
+
+def _add_worker_data_from_tuple(
+    worker: FunctionWorker, return_type, source=None
+):
+    from napari._qt._qapp_model.injection._qprocessors import (
+        _add_layer_data_tuples_to_viewer,
+    )
+
+    cb = _add_layer_data_tuples_to_viewer
+    worker.signals.returned.connect(
+        partial(cb, return_type=return_type, source=source)
+    )
+
+
+def register_threadworker_processors():
     from functools import partial
 
     import magicgui
 
-    from .. import layers, types
-    from ..types import LayerDataTuple
-    from ..utils import _magicgui as _mgui
+    from napari import layers, types
+    from napari._app_model import get_app_model
+    from napari.types import LayerDataTuple
+    from napari.utils import _magicgui as _mgui
 
-    for _type in (LayerDataTuple, List[LayerDataTuple]):
-        magicgui.register_type(
-            FunctionWorker[_type], return_callback=_mgui.add_worker_data
+    app = get_app_model()
+
+    for _type in (LayerDataTuple, list[LayerDataTuple]):
+        t = FunctionWorker[_type]
+        magicgui.register_type(t, return_callback=_mgui.add_worker_data)
+        app.injection_store.register(
+            processors={t: _add_worker_data_from_tuple}
         )
     for layer_name in layers.NAMES:
         _type = getattr(types, f'{layer_name.title()}Data')
+        t = FunctionWorker[_type]
         magicgui.register_type(
-            FunctionWorker[_type],
+            t,
             return_callback=partial(_mgui.add_worker_data, _from_tuple=False),
         )
-
-
-_register()
+        app.injection_store.register(processors={t: _add_worker_data})
