@@ -31,10 +31,13 @@ Notes for using the plugin-related fixtures here:
 
 from __future__ import annotations
 
+import contextlib
 import os
 import sys
+import threading
 from concurrent.futures import ThreadPoolExecutor
 from contextlib import suppress
+from functools import partial
 from itertools import chain
 from multiprocessing.pool import ThreadPool
 from pathlib import Path
@@ -71,7 +74,7 @@ if os.getenv('CI') and sys.platform.startswith('linux'):
         xauth.touch()
 
 
-@pytest.fixture()
+@pytest.fixture
 def layer_data_and_types():
     """Fixture that provides some layers and filenames
 
@@ -153,7 +156,7 @@ def layer(request):
     return None
 
 
-@pytest.fixture()
+@pytest.fixture
 def layers():
     """Fixture that supplies a layers list for testing.
 
@@ -254,7 +257,7 @@ def _auto_shutdown_dask_threadworkers():
 HistoryManager.enabled = False
 
 
-@pytest.fixture()
+@pytest.fixture
 def napari_svg_name():
     """the plugin name changes with npe2 to `napari-svg` from `svg`."""
     from importlib.metadata import version
@@ -274,13 +277,13 @@ def npe2pm_(npe2pm, monkeypatch):
     return npe2pm
 
 
-@pytest.fixture()
+@pytest.fixture
 def builtins(npe2pm_: TestPluginManager):
     with npe2pm_.tmp_plugin(package='napari') as plugin:
         yield plugin
 
 
-@pytest.fixture()
+@pytest.fixture
 def tmp_plugin(npe2pm_: TestPluginManager):
     with npe2pm_.tmp_plugin() as plugin:
         plugin.manifest.package_metadata = PackageMetadata(  # type: ignore[call-arg]
@@ -290,6 +293,140 @@ def tmp_plugin(npe2pm_: TestPluginManager):
         yield plugin
 
 
+@pytest.fixture
+def viewer_model():
+    from napari.components import ViewerModel
+
+    return ViewerModel()
+
+
+@pytest.fixture
+def qt_viewer_(qtbot, viewer_model, monkeypatch):
+    from napari._qt.qt_viewer import QtViewer
+
+    viewer = QtViewer(viewer_model)
+
+    original_controls = viewer.__class__.controls.fget
+    original_layers = viewer.__class__.layers.fget
+    original_layer_buttons = viewer.__class__.layerButtons.fget
+    original_viewer_buttons = viewer.__class__.viewerButtons.fget
+    original_dock_layer_list = viewer.__class__.dockLayerList.fget
+    original_dock_layer_controls = viewer.__class__.dockLayerControls.fget
+    original_dock_console = viewer.__class__.dockConsole.fget
+    original_dock_performance = viewer.__class__.dockPerformance.fget
+
+    def hide_widget(widget):
+        widget.hide()
+
+    def hide_and_clear_qt_viewer(viewer: QtViewer):
+        viewer._instances.clear()
+        viewer.hide()
+
+    def patched_controls(self):
+        if self._controls is None:
+            self._controls = original_controls(self)
+            qtbot.addWidget(self._controls, before_close_func=hide_widget)
+        return self._controls
+
+    def patched_layers(self):
+        if self._layers is None:
+            self._layers = original_layers(self)
+            qtbot.addWidget(self._layers, before_close_func=hide_widget)
+        return self._layers
+
+    def patched_layer_buttons(self):
+        if self._layersButtons is None:
+            self._layersButtons = original_layer_buttons(self)
+            qtbot.addWidget(self._layersButtons, before_close_func=hide_widget)
+        return self._layersButtons
+
+    def patched_viewer_buttons(self):
+        if self._viewerButtons is None:
+            self._viewerButtons = original_viewer_buttons(self)
+            qtbot.addWidget(self._viewerButtons, before_close_func=hide_widget)
+        return self._viewerButtons
+
+    def patched_dock_layer_list(self):
+        if self._dockLayerList is None:
+            self._dockLayerList = original_dock_layer_list(self)
+            qtbot.addWidget(self._dockLayerList, before_close_func=hide_widget)
+        return self._dockLayerList
+
+    def patched_dock_layer_controls(self):
+        if self._dockLayerControls is None:
+            self._dockLayerControls = original_dock_layer_controls(self)
+            qtbot.addWidget(
+                self._dockLayerControls, before_close_func=hide_widget
+            )
+        return self._dockLayerControls
+
+    def patched_dock_console(self):
+        if self._dockConsole is None:
+            self._dockConsole = original_dock_console(self)
+            qtbot.addWidget(self._dockConsole, before_close_func=hide_widget)
+        return self._dockConsole
+
+    def patched_dock_performance(self):
+        if self._dockPerformance is None:
+            self._dockPerformance = original_dock_performance(self)
+            qtbot.addWidget(
+                self._dockPerformance, before_close_func=hide_widget
+            )
+        return self._dockPerformance
+
+    monkeypatch.setattr(
+        viewer.__class__, 'controls', property(patched_controls)
+    )
+    monkeypatch.setattr(viewer.__class__, 'layers', property(patched_layers))
+    monkeypatch.setattr(
+        viewer.__class__, 'layerButtons', property(patched_layer_buttons)
+    )
+    monkeypatch.setattr(
+        viewer.__class__, 'viewerButtons', property(patched_viewer_buttons)
+    )
+    monkeypatch.setattr(
+        viewer.__class__, 'dockLayerList', property(patched_dock_layer_list)
+    )
+    monkeypatch.setattr(
+        viewer.__class__,
+        'dockLayerControls',
+        property(patched_dock_layer_controls),
+    )
+    monkeypatch.setattr(
+        viewer.__class__, 'dockConsole', property(patched_dock_console)
+    )
+    monkeypatch.setattr(
+        viewer.__class__, 'dockPerformance', property(patched_dock_performance)
+    )
+
+    qtbot.addWidget(viewer, before_close_func=hide_and_clear_qt_viewer)
+    return viewer
+
+
+@pytest.fixture
+def qt_viewer(qt_viewer_):
+    """We created `qt_viewer_` fixture to allow modifying qt_viewer
+    if module-level-specific modifications are necessary.
+    For example, in `test_qt_viewer.py`.
+    """
+    return qt_viewer_
+
+
+@pytest.fixture(autouse=True)
+def _clear_cached_action_injection():
+    """Automatically clear cached property `Action.injected`.
+
+    Allows action manager actions to be injected using current provider/processors
+    and dependencies. See #7219 for details.
+    To be removed after ActionManager deprecation.
+    """
+    from napari.utils.action_manager import action_manager
+
+    for action in action_manager._actions.values():
+        if 'injected' in action.__dict__:
+            del action.__dict__['injected']
+
+
 def _event_check(instance):
     def _prepare_check(name, no_event_):
         def check(instance, no_event=no_event_):
@@ -389,7 +526,7 @@ def _disable_notification_dismiss_timer(monkeypatch):
         monkeypatch.setattr(NapariQtNotification, 'FADE_OUT_RATE', 0)
 
 
-@pytest.fixture()
+@pytest.fixture
 def single_threaded_executor():
     executor = ThreadPoolExecutor(max_workers=1)
     yield executor
@@ -423,26 +560,60 @@ def _get_calling_place(depth=1):  # pragma: no cover
     return result
 
 
-@pytest.fixture()
+@pytest.fixture
 def _dangling_qthreads(monkeypatch, qtbot, request):
     from qtpy.QtCore import QThread
 
     base_start = QThread.start
     thread_dict = WeakKeyDictionary()
+    base_constructor = QThread.__init__
+
+    def run_with_trace(self):  # pragma: no cover
+        """
+        QThread.run but adding execution to sys.settrace when measuring coverage.
+
+        See https://github.com/nedbat/coveragepy/issues/686#issuecomment-634932753
+        and `init_with_trace`. When running QThreads during testing, we monkeypatch
+        the QThread constructor and run methods with traceable equivalents.
+        """
+        if 'coverage' in sys.modules:
+            # https://github.com/nedbat/coveragepy/issues/686#issuecomment-634932753
+            sys.settrace(threading._trace_hook)
+        self._base_run()
+
+    def init_with_trace(self, *args, **kwargs):
+        """Constructor for QThread adding tracing for coverage measurements.
+
+        Functions running in QThreads don't get measured by coverage.py, see
+        https://github.com/nedbat/coveragepy/issues/686. Therefore, we will
+        monkeypatch the constructor to add to the thread to `sys.settrace` when
+        we call `run` and `coverage` is in `sys.modules`.
+        """
+        base_constructor(self, *args, **kwargs)
+        self._base_run = self.run
+        self.run = partial(run_with_trace, self)
+
     # dict of threads that have been started but not yet terminated
 
     if 'disable_qthread_start' in request.keywords:
 
-        def my_start(self, priority=QThread.InheritPriority):
-            """dummy function to prevent thread start"""
+        def start_with_save_reference(self, priority=QThread.InheritPriority):
+            """Dummy function to prevent thread starts."""
 
     else:
 
-        def my_start(self, priority=QThread.InheritPriority):
+        def start_with_save_reference(self, priority=QThread.InheritPriority):
+            """Thread start function with logs to detect hanging threads.
+
+            Saves a weak reference to the thread and detects hanging threads,
+            as well as where the threads were started.
+            """
             thread_dict[self] = _get_calling_place()
             base_start(self, priority)
 
-    monkeypatch.setattr(QThread, 'start', my_start)
+    monkeypatch.setattr(QThread, 'start', start_with_save_reference)
+    monkeypatch.setattr(QThread, '__init__', init_with_trace)
+
     yield
 
     dangling_threads_li = []
@@ -483,7 +654,7 @@ def _dangling_qthreads(monkeypatch, qtbot, request):
     )
 
 
-@pytest.fixture()
+@pytest.fixture
 def _dangling_qthread_pool(monkeypatch, request):
     from qtpy.QtCore import QThreadPool
 
@@ -539,7 +710,7 @@ def _dangling_qthread_pool(monkeypatch, request):
     )
 
 
-@pytest.fixture()
+@pytest.fixture
 def _dangling_qtimers(monkeypatch, request):
     from qtpy.QtCore import QTimer
 
@@ -645,7 +816,7 @@ def _flush_mock(self):
     """There are no waiting events."""
 
 
-@pytest.fixture()
+@pytest.fixture
 def _disable_throttling(monkeypatch):
     """Disable qthrottler from superqt.
 
@@ -662,7 +833,7 @@ def _disable_throttling(monkeypatch):
     )
 
 
-@pytest.fixture()
+@pytest.fixture
 def _dangling_qanimations(monkeypatch, request):
     from qtpy.QtCore import QPropertyAnimation
 
@@ -708,15 +879,139 @@ def _dangling_qanimations(monkeypatch, request):
     )
 
 
+with contextlib.suppress(ImportError):
+    # in headless test suite we don't have Qt bindings
+    # So we cannot inherit from QtBot and declare the fixture
+
+    from pytestqt.qtbot import QtBot
+
+    class QtBotWithOnCloseRenaming(QtBot):
+        """Modified QtBot that renames widgets when closing them in tests.
+
+        After a test ends that uses QtBot, all instantiated widgets added to
+        the bot have their name changed to 'handled_widget'. This allows us to
+        detect leaking widgets at the end of a test run, and avoid the
+        segmentation faults that often result from such leaks. [1]_
+
+        See Also
+        --------
+        `_find_dangling_widgets`: fixture that finds all widgets that have not
+        been renamed to 'handled_widget'.
+
+        References
+        ----------
+        .. [1] https://czaki.github.io/blog/2024/09/16/preventing-segfaults-in-test-suite-that-has-qt-tests/
+        """
+
+        def addWidget(self, widget, *, before_close_func=None):
+            if widget.objectName() == '':
+                # object does not have a name, so we can set it
+                widget.setObjectName('handled_widget')
+                before_close_func_ = before_close_func
+            elif before_close_func is None:
+                # there is no custom teardown function,
+                # so we provide one that will set object name
+
+                def before_close_func_(w):
+                    w.setObjectName('handled_widget')
+            else:
+                # user provided custom teardown function,
+                # so we need to wrap it to set object name
+
+                def before_close_func_(w):
+                    before_close_func(w)
+                    w.setObjectName('handled_widget')
+
+            super().addWidget(widget, before_close_func=before_close_func_)
+
+    @pytest.fixture
+    def qtbot(qapp, request):  # pragma: no cover
+        """Fixture to create a QtBotWithOnCloseRenaming instance for testing.
+
+        Make sure to call addWidget for each top-level widget you create to
+        ensure that they are properly closed after the test ends.
+
+        The `qapp` fixture is used to ensure that the QApplication is created
+        before, so we need it, even without using it directly in this fixture.
+        """
+        return QtBotWithOnCloseRenaming(request)
+
+
+@pytest.fixture
+def _find_dangling_widgets(request, qtbot):
+    yield
+
+    from qtpy.QtWidgets import QApplication
+
+    from napari._qt.qt_main_window import _QtMainWindow
+
+    top_level_widgets = QApplication.topLevelWidgets()
+
+    viewer_weak_set = getattr(request.node, '_viewer_weak_set', set())
+
+    problematic_widgets = []
+
+    for widget in top_level_widgets:
+        if widget.parent() is not None:
+            continue
+        if (
+            isinstance(widget, _QtMainWindow)
+            and widget._qt_viewer.viewer in viewer_weak_set
+        ):
+            continue
+
+        if widget.__class__.__module__.startswith('qtconsole'):
+            continue
+
+        if widget.objectName() == 'handled_widget':
+            continue
+
+        if widget.__class__.__name__ == 'CanvasBackendDesktop':
+            # TODO: we don't understand why this class leaks in
+            #  napari/_tests/test_sys_info.py, so we make an exception
+            #  here and we don't raise when this class leaks.
+            continue
+
+        problematic_widgets.append(widget)
+
+    if problematic_widgets:
+        text = '\n'.join(
+            f'Widget: {widget} of type {type(widget)} with name {widget.objectName()}'
+            for widget in problematic_widgets
+        )
+        raise RuntimeError(f'Found dangling widgets:\n{text}')
+    for widget in problematic_widgets:
+        widget.setObjectName('handled_widget')
+
+
 def pytest_runtest_setup(item):
+    """Add Qt leak detection fixtures *only* in tests using the qapp fixture.
+
+    Because we have headless test suite that does not include Qt, we cannot
+    simply use `@pytest.fixture(autouse=True)` on all our fixtures for
+    detecting leaking Qt objects.
+
+    Instead, here we detect whether the `qapp` fixture is being used, detecting
+    tests that use Qt and need to be checked for Qt objects leaks.
+
+    A note to maintainers: tests *may* attempt to use Qt classes but not use
+    the `qapp` fixture. This is BAD, and may cause Qt failures to be reported
+    far away from the problematic code or test. If you find any tests
+    instantiating Qt objects but not using qapp or qtbot, please submit a PR
+    adding the qtbot fixture and adding any top-level Qt widgets with::
+
+        qtbot.addWidget(widget_instance)
+
+    """
+
     if 'qapp' in item.fixturenames:
         # here we do autouse for dangling fixtures only if qapp is used
         if 'qtbot' not in item.fixturenames:
             # for proper waiting for threads to finish
             item.fixturenames.append('qtbot')
-
         item.fixturenames.extend(
             [
+                '_find_dangling_widgets',
                 '_dangling_qthread_pool',
                 '_dangling_qanimations',
                 '_dangling_qthreads',
