@@ -1,14 +1,16 @@
 from functools import lru_cache
 
-from npe2 import PackageMetadata
-from npe2 import PluginManager as _PluginManager
-from npe2 import PluginManifest
+from npe2 import (
+    PluginManager as _PluginManager,
+)
 
-from ..settings import get_settings
-from . import _npe2
-from ._plugin_manager import NapariPluginManager
+from napari.plugins import _npe2
+from napari.plugins._plugin_manager import NapariPluginManager
+from napari.settings import get_settings
 
-__all__ = ("plugin_manager", "menu_item_template")
+__all__ = ('plugin_manager', 'menu_item_template')
+
+from napari.utils.theme import _install_npe2_themes
 
 #: Template to use for namespacing a plugin item in the menu bar
 # widget_name (plugin_name)
@@ -20,7 +22,7 @@ plugin_manager = NapariPluginManager()
 
 
 @lru_cache  # only call once
-def _initialize_plugins():
+def _initialize_plugins() -> None:
     _npe2pm = _PluginManager.instance()
 
     settings = get_settings()
@@ -38,15 +40,6 @@ def _initialize_plugins():
     _npe2pm.events.plugins_registered.connect(_npe2.on_plugins_registered)
     _npe2pm.discover(include_npe1=settings.plugins.use_npe2_adaptor)
 
-    # this is a workaround for the fact that briefcase does not seem to include
-    # napari's entry_points.txt in the bundled app, so the builtin plugins
-    # don't get detected.  So we just register it manually.  This could
-    # potentially be removed when we move to a different bundle strategy
-    if 'napari' not in _npe2pm._manifests:
-        mf = PluginManifest.from_distribution('napari')
-        mf.package_metadata = PackageMetadata.for_package('napari')
-        _npe2pm.register(mf)
-
     # Disable plugins listed as disabled in settings, or detected in npe2
     _from_npe2 = {m.name for m in _npe2pm.iter_manifests()}
     if 'napari' in _from_npe2:
@@ -60,3 +53,5 @@ def _initialize_plugins():
         plugin_manager.discover = lambda *a, **k: None
     else:
         plugin_manager._initialize()
+
+    _install_npe2_themes()
