@@ -1,20 +1,20 @@
-from napari._qt.qt_event_loop import get_app, run
+from napari._qt.qt_event_loop import get_app, get_qapp, run
 from napari._qt.qt_main_window import Window
 from napari._qt.qt_resources import get_current_stylesheet, get_stylesheet
 from napari._qt.qt_viewer import QtViewer
 from napari._qt.widgets.qt_tooltip import QtToolTipLabel
-from napari._qt.widgets.qt_viewer_buttons import QtStateButton, QtViewerButtons
+from napari._qt.widgets.qt_viewer_buttons import QtViewerButtons
 from napari.qt.threading import create_worker, thread_worker
 
 __all__ = (
     'create_worker',
-    'QtStateButton',
     'QtToolTipLabel',
     'QtViewer',
     'QtViewerButtons',
     'thread_worker',
     'Window',
     'get_app',
+    'get_qapp',
     'get_stylesheet',
     'get_current_stylesheet',
     'run',
