@@ -5,10 +5,10 @@ This is used by the translation loader as the settings models require using
 the translator before the settings manager is created.
 """
 
-from appdirs import user_config_dir
+import os
 
-_FILENAME = "settings.yaml"
-_APPNAME = "Napari"
-_APPAUTHOR = "Napari"
-_DEFAULT_LOCALE = "en"
-_DEFAULT_CONFIG_PATH = user_config_dir(_APPNAME, _APPAUTHOR, _FILENAME)
+from napari.utils._appdirs import user_config_dir
+
+_FILENAME = 'settings.yaml'
+_DEFAULT_LOCALE = 'en'
+_DEFAULT_CONFIG_PATH = os.path.join(user_config_dir(), _FILENAME)
